/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.builders.addon;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.VertexBuffer;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Vec3d;

import net.minecraftforge.client.model.ModelLoader;

import buildcraft.core.marker.volume.IFastAddonRenderer;

public class AddonRendererFillerPlanner implements IFastAddonRenderer<AddonFillerPlanner> {
    @Override
    public void renderAddonFast(AddonFillerPlanner addon, EntityPlayer player, float partialTicks, VertexBuffer vb) {
        if (addon.buildingInfo == null) {
            return;
        }
        Minecraft.func_71410_x().field_71424_I.func_76320_a("filler_planner");

        Minecraft.func_71410_x().field_71424_I.func_76320_a("iter");
        List<BlockPos> list = StreamSupport.stream(
            BlockPos.func_177975_b(addon.buildingInfo.box.min(), addon.buildingInfo.box.max()).spliterator(),
            false
        )
            .filter(blockPos ->
                addon.buildingInfo.getSnapshot().data.get(
                    addon.buildingInfo.getSnapshot().posToIndex(
                        addon.buildingInfo.fromWorld(blockPos)
                    )
                )
            )
            .filter(player.field_70170_p::func_175623_d)
            .map(BlockPos.MutableBlockPos::func_185334_h)
            .collect(Collectors.toCollection(ArrayList::new));
        Minecraft.func_71410_x().field_71424_I.func_76319_b();

        Minecraft.func_71410_x().field_71424_I.func_76320_a("sort");
        list.sort(Comparator.<BlockPos>comparingDouble(p -> player.func_174791_d().func_72436_e(new Vec3d(p))).reversed());
        Minecraft.func_71410_x().field_71424_I.func_76319_b();

        Minecraft.func_71410_x().field_71424_I.func_76320_a("render");
        for (BlockPos p : list) {
            AxisAlignedBB bb = new AxisAlignedBB(p, p.func_177982_a(1, 1, 1)).func_186662_g(-0.1);
            TextureAtlasSprite s = ModelLoader.White.INSTANCE;

            vb.func_181662_b(bb.field_72340_a, bb.field_72337_e, bb.field_72339_c).func_181669_b(204, 204, 204, 127).func_187315_a(s.func_94209_e(), s.func_94206_g()).func_187314_a(240, 0).func_181675_d();
            vb.func_181662_b(bb.field_72336_d, bb.field_72337_e, bb.field_72339_c).func_181669_b(204, 204, 204, 127).func_187315_a(s.func_94209_e(), s.func_94210_h()).func_187314_a(240, 0).func_181675_d();
            vb.func_181662_b(bb.field_72336_d, bb.field_72338_b, bb.field_72339_c).func_181669_b(204, 204, 204, 127).func_187315_a(s.func_94212_f(), s.func_94210_h()).func_187314_a(240, 0).func_181675_d();
            vb.func_181662_b(bb.field_72340_a, bb.field_72338_b, bb.field_72339_c).func_181669_b(204, 204, 204, 127).func_187315_a(s.func_94212_f(), s.func_94206_g()).func_187314_a(240, 0).func_181675_d();

            vb.func_181662_b(bb.field_72340_a, bb.field_72338_b, bb.field_72334_f).func_181669_b(204, 204, 204, 127).func_187315_a(s.func_94209_e(), s.func_94206_g()).func_187314_a(240, 0).func_181675_d();
            vb.func_181662_b(bb.field_72336_d, bb.field_72338_b, bb.field_72334_f).func_181669_b(204, 204, 204, 127).func_187315_a(s.func_94209_e(), s.func_94210_h()).func_187314_a(240, 0).func_181675_d();
            vb.func_181662_b(bb.field_72336_d, bb.field_72337_e, bb.field_72334_f).func_181669_b(204, 204, 204, 127).func_187315_a(s.func_94212_f(), s.func_94210_h()).func_187314_a(240, 0).func_181675_d();
            vb.func_181662_b(bb.field_72340_a, bb.field_72337_e, bb.field_72334_f).func_181669_b(204, 204, 204, 127).func_187315_a(s.func_94212_f(), s.func_94206_g()).func_187314_a(240, 0).func_181675_d();

            vb.func_181662_b(bb.field_72340_a, bb.field_72338_b, bb.field_72339_c).func_181669_b(127, 127, 127, 127).func_187315_a(s.func_94209_e(), s.func_94206_g()).func_187314_a(240, 0).func_181675_d();
            vb.func_181662_b(bb.field_72336_d, bb.field_72338_b, bb.field_72339_c).func_181669_b(127, 127, 127, 127).func_187315_a(s.func_94209_e(), s.func_94210_h()).func_187314_a(240, 0).func_181675_d();
            vb.func_181662_b(bb.field_72336_d, bb.field_72338_b, bb.field_72334_f).func_181669_b(127, 127, 127, 127).func_187315_a(s.func_94212_f(), s.func_94210_h()).func_187314_a(240, 0).func_181675_d();
            vb.func_181662_b(bb.field_72340_a, bb.field_72338_b, bb.field_72334_f).func_181669_b(127, 127, 127, 127).func_187315_a(s.func_94212_f(), s.func_94206_g()).func_187314_a(240, 0).func_181675_d();

            vb.func_181662_b(bb.field_72340_a, bb.field_72337_e, bb.field_72334_f).func_181669_b(255, 255, 255, 127).func_187315_a(s.func_94209_e(), s.func_94206_g()).func_187314_a(240, 0).func_181675_d();
            vb.func_181662_b(bb.field_72336_d, bb.field_72337_e, bb.field_72334_f).func_181669_b(255, 255, 255, 127).func_187315_a(s.func_94209_e(), s.func_94210_h()).func_187314_a(240, 0).func_181675_d();
            vb.func_181662_b(bb.field_72336_d, bb.field_72337_e, bb.field_72339_c).func_181669_b(255, 255, 255, 127).func_187315_a(s.func_94212_f(), s.func_94210_h()).func_187314_a(240, 0).func_181675_d();
            vb.func_181662_b(bb.field_72340_a, bb.field_72337_e, bb.field_72339_c).func_181669_b(255, 255, 255, 127).func_187315_a(s.func_94212_f(), s.func_94206_g()).func_187314_a(240, 0).func_181675_d();

            vb.func_181662_b(bb.field_72340_a, bb.field_72338_b, bb.field_72334_f).func_181669_b(153, 153, 153, 127).func_187315_a(s.func_94209_e(), s.func_94206_g()).func_187314_a(240, 0).func_181675_d();
            vb.func_181662_b(bb.field_72340_a, bb.field_72337_e, bb.field_72334_f).func_181669_b(153, 153, 153, 127).func_187315_a(s.func_94209_e(), s.func_94210_h()).func_187314_a(240, 0).func_181675_d();
            vb.func_181662_b(bb.field_72340_a, bb.field_72337_e, bb.field_72339_c).func_181669_b(153, 153, 153, 127).func_187315_a(s.func_94212_f(), s.func_94210_h()).func_187314_a(240, 0).func_181675_d();
            vb.func_181662_b(bb.field_72340_a, bb.field_72338_b, bb.field_72339_c).func_181669_b(153, 153, 153, 127).func_187315_a(s.func_94212_f(), s.func_94206_g()).func_187314_a(240, 0).func_181675_d();

            vb.func_181662_b(bb.field_72336_d, bb.field_72338_b, bb.field_72339_c).func_181669_b(153, 153, 153, 127).func_187315_a(s.func_94209_e(), s.func_94206_g()).func_187314_a(240, 0).func_181675_d();
            vb.func_181662_b(bb.field_72336_d, bb.field_72337_e, bb.field_72339_c).func_181669_b(153, 153, 153, 127).func_187315_a(s.func_94209_e(), s.func_94210_h()).func_187314_a(240, 0).func_181675_d();
            vb.func_181662_b(bb.field_72336_d, bb.field_72337_e, bb.field_72334_f).func_181669_b(153, 153, 153, 127).func_187315_a(s.func_94212_f(), s.func_94210_h()).func_187314_a(240, 0).func_181675_d();
            vb.func_181662_b(bb.field_72336_d, bb.field_72338_b, bb.field_72334_f).func_181669_b(153, 153, 153, 127).func_187315_a(s.func_94212_f(), s.func_94206_g()).func_187314_a(240, 0).func_181675_d();
        }
        Minecraft.func_71410_x().field_71424_I.func_76319_b();

        Minecraft.func_71410_x().field_71424_I.func_76319_b();
    }
}
