/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.core.block;

import java.util.Random;
import java.util.function.Supplier;

import net.minecraft.block.SoundType;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.NonNullList;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

import buildcraft.api.enums.EnumSpring;
import buildcraft.api.properties.BuildCraftProperties;

import buildcraft.lib.block.BlockBCBase_Neptune;
import buildcraft.lib.misc.data.XorShift128Random;

public class BlockSpring extends BlockBCBase_Neptune {
    public static final IProperty<EnumSpring> SPRING_TYPE = BuildCraftProperties.SPRING_TYPE;

    public static final XorShift128Random rand = new XorShift128Random();

    public BlockSpring(String id) {
        super(Material.field_151576_e, id);
        func_149722_s();
        func_149752_b(6000000.0F);
        func_149672_a(SoundType.field_185851_d);

        func_149649_H();
        func_149675_a(true);
        func_180632_j(func_176223_P().func_177226_a(SPRING_TYPE, EnumSpring.WATER));
    }

    // BlockState

    @Override
    protected BlockStateContainer func_180661_e() {
        return new BlockStateContainer(this, SPRING_TYPE);
    }

    @Override
    public int func_176201_c(IBlockState state) {
        return state.func_177229_b(SPRING_TYPE).ordinal();
    }

    @Override
    public IBlockState func_176203_a(int meta) {
        if (meta == EnumSpring.OIL.ordinal()) {
            return func_176223_P().func_177226_a(SPRING_TYPE, EnumSpring.OIL);
        } else {
            return func_176223_P().func_177226_a(SPRING_TYPE, EnumSpring.WATER);
        }
    }

    // Other

    @Override
    public void func_149666_a(Item itemIn, CreativeTabs tab, NonNullList<ItemStack> list) {
        for (EnumSpring type : EnumSpring.VALUES) {
            list.add(new ItemStack(this, 1, type.ordinal()));
        }
    }

    @Override
    public int func_180651_a(IBlockState state) {
        return state.func_177229_b(SPRING_TYPE).ordinal();
    }

    @Override
    public void func_180650_b(World world, BlockPos pos, IBlockState state, Random random) {
        generateSpringBlock(world, pos, state);
    }

    @Override
    public boolean hasTileEntity(IBlockState state) {
        return state.func_177229_b(SPRING_TYPE).tileConstructor != null;
    }

    @Override
    public TileEntity createTileEntity(World world, IBlockState state) {
        Supplier<TileEntity> constructor = state.func_177229_b(SPRING_TYPE).tileConstructor;
        if (constructor != null) {
            return constructor.get();
        }
        return null;
    }
    
    // @Override
    // public void onNeighborBlockChange(World world, int x, int y, int z, int blockid) {
    // assertSpring(world, x, y, z);
    // }

    @Override
    public void func_176213_c(World world, BlockPos pos, IBlockState state) {
        super.func_176213_c(world, pos, state);
        world.func_175684_a(pos, this, state.func_177229_b(SPRING_TYPE).tickRate);
    }

    private void generateSpringBlock(World world, BlockPos pos, IBlockState state) {
        EnumSpring spring = state.func_177229_b(SPRING_TYPE);
        world.func_175684_a(pos, this, spring.tickRate);
        if (!spring.canGen || spring.liquidBlock == null) {
            return;
        }
        if (!world.func_175623_d(pos.func_177984_a())) {
            return;
        }
        if (spring.chance != -1 && rand.nextInt(spring.chance) != 0) {
            return;
        }
        world.func_175656_a(pos.func_177984_a(), spring.liquidBlock);
    }

    // Prevents updates on chunk generation
    // @Override
    // public boolean func_149698_L() {
    // return false;
    // }
}
