/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.lib.client.render.laser;

import org.lwjgl.opengl.GL11;

import net.minecraft.client.renderer.GLAllocation;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.renderer.OpenGlHelper;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.VertexBuffer;

public abstract class LaserCompiledList {
    public abstract void render();

    public abstract void delete();

    public static class Builder implements ILaserRenderer {
        private final Tessellator tess;
        private final VertexBuffer buffer;
        private final boolean useColour;

        public Builder(boolean useNormalColour) {
            this.useColour = useNormalColour;
            tess = Tessellator.func_178181_a();
            buffer = tess.func_178180_c();
            buffer.func_181668_a(GL11.GL_QUADS, useNormalColour ? LaserRenderer_BC8.FORMAT_ALL : LaserRenderer_BC8.FORMAT_LESS);
        }

        @Override
        public void vertex(double x, double y, double z, double u, double v, int lmap, float nx, float ny, float nz, float diffuse) {
            buffer.func_181662_b(x, y, z);
            buffer.func_187315_a(u, v);
            buffer.func_187314_a((lmap >> 16) & 0xFFFF, lmap & 0xFFFF);
            if (useColour) {
                buffer.func_181666_a(diffuse, diffuse, diffuse, 1.0f);
            }
            buffer.func_181675_d();
        }

        public LaserCompiledList build() {
            if (OpenGlHelper.func_176075_f()) {
                net.minecraft.client.renderer.vertex.VertexBuffer vb = new net.minecraft.client.renderer.vertex.VertexBuffer(this.buffer.func_178973_g());
                buffer.func_178977_d();
                buffer.func_178965_a();
                vb.func_181722_a(buffer.func_178966_f());
                return new Vbo(useColour, vb);
            } else {
                int glList = GLAllocation.func_74526_a(1);
                GL11.glNewList(glList, GL11.GL_COMPILE);
                tess.func_78381_a();
                GL11.glEndList();
                return new GlList(glList);
            }
        }
    }

    private static class GlList extends LaserCompiledList {
        private final int glListId;

        private GlList(int glListId) {
            this.glListId = glListId;
        }

        @Override
        public void render() {
            GL11.glCallList(glListId);
        }

        @Override
        public void delete() {
            GL11.glDeleteLists(glListId, 1);
        }
    }

    private static class Vbo extends LaserCompiledList {
        private final boolean useColour;
        private final net.minecraft.client.renderer.vertex.VertexBuffer buffer;

        private Vbo(boolean useColour, net.minecraft.client.renderer.vertex.VertexBuffer buffer) {
            this.useColour = useColour;
            this.buffer = buffer;
        }

        @Override
        public void render() {
            final int stride = useColour ? 28 : 24;

            buffer.func_177359_a();
            GlStateManager.func_187410_q(GL11.GL_VERTEX_ARRAY);
            GlStateManager.func_187420_d(3, GL11.GL_FLOAT, stride, 0);

            GlStateManager.func_187410_q(GL11.GL_TEXTURE_COORD_ARRAY);
            GlStateManager.func_187405_c(2, GL11.GL_FLOAT, stride, 12);

            OpenGlHelper.func_77472_b(OpenGlHelper.field_77476_b);
            GlStateManager.func_187410_q(GL11.GL_TEXTURE_COORD_ARRAY);
            GlStateManager.func_187405_c(2, GL11.GL_SHORT, stride, 20);
            OpenGlHelper.func_77472_b(OpenGlHelper.field_77478_a);

            if (useColour) {
                GlStateManager.func_187410_q(GL11.GL_COLOR_ARRAY);
                GlStateManager.func_187406_e(4, GL11.GL_UNSIGNED_BYTE, stride, 24);
            }

            buffer.func_177358_a(GL11.GL_QUADS);
            buffer.func_177361_b();

            GlStateManager.func_187429_p(GL11.GL_VERTEX_ARRAY);
            GlStateManager.func_187429_p(GL11.GL_TEXTURE_COORD_ARRAY);
            OpenGlHelper.func_77472_b(OpenGlHelper.field_77476_b);
            GlStateManager.func_187429_p(GL11.GL_TEXTURE_COORD_ARRAY);
            OpenGlHelper.func_77472_b(OpenGlHelper.field_77478_a);

            if (useColour) {
                GlStateManager.func_187429_p(GL11.GL_COLOR_ARRAY);
                GlStateManager.func_179131_c(1, 1, 1, 1);
            }
        }

        @Override
        public void delete() {
            buffer.func_177362_c();
        }
    }
}
