/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.lib.client.sprite;

import java.io.IOException;
import java.util.List;

import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.texture.PngSizeInfo;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.client.renderer.texture.TextureUtil;
import net.minecraft.client.resources.IResource;
import net.minecraft.client.resources.IResourceManager;
import net.minecraft.profiler.Profiler;
import net.minecraft.util.ResourceLocation;

import net.minecraftforge.fml.client.FMLClientHandler;

/** Provides the basic implementation for */
public abstract class AtlasSpriteSwappable extends TextureAtlasSprite {
    private TextureAtlasSprite current;
    private boolean needsSwapping = true;

    public AtlasSpriteSwappable(String baseName) {
        super(baseName);
    }

    @Override
    public boolean func_130098_m() {
        return true;
    }

    @Override
    public void func_94219_l() {
        if (current == null) {
            func_94217_a(Minecraft.func_71410_x().func_147117_R().func_174944_f());
            return;
        }
        Profiler p = Minecraft.func_71410_x().field_71424_I;
        p.startSection(getClass());
        if (needsSwapping) {
            p.func_76320_a("copy");
            current.func_94217_a(this);
            p.func_76319_b();
        }
        if (current.func_130098_m()) {
            p.func_76320_a("update");
            p.func_76320_a(func_94215_i());
            current.func_94219_l();
            p.func_76319_b();
            p.func_76319_b();
        } else if (needsSwapping) {
            p.func_76320_a("swap");
            TextureUtil.func_147955_a(current.func_147965_a(0), current.func_94211_a(), current.func_94216_b(), current.func_130010_a(), current.func_110967_i(), false, false);
            p.func_76319_b();
        }
        needsSwapping = false;
        p.func_76319_b();
    }

    public boolean swapWith(TextureAtlasSprite other) {
        if (current != other && (current == null || other != null)) {
            current = other;
            if (field_130223_c == 0) {
                this.field_130223_c = other.func_94211_a();
                this.field_130224_d = other.func_94216_b();
            }
            func_147963_d(Minecraft.func_71410_x().field_71474_y.field_151442_I);
            needsSwapping = true;
            return true;
        }
        return false;
    }

    public void reload(ResourceLocation from) {
        load(Minecraft.func_71410_x().func_110442_L(), from);
    }

    /** Actually loads the given location. Note that subclasses should override this, and possibly call
     * {@link #loadSprite(IResourceManager, String, ResourceLocation, boolean)} to load all of the possible variants. */
    @Override
    public boolean load(IResourceManager manager, ResourceLocation location) {
        TextureAtlasSprite sprite = loadSprite(manager, super.func_94215_i(), location, true);
        if (sprite != null) {
            swapWith(sprite);
        }
        return false;
    }

    public static AtlasSpriteDirect loadSprite(String name, ResourceLocation location, boolean careIfMissing) {
        return loadSprite(Minecraft.func_71410_x().func_110442_L(), name, location, careIfMissing);
    }

    public static AtlasSpriteDirect loadSprite(IResourceManager manager, String name, ResourceLocation location, boolean careIfMissing) {
        // Load the initial variant
        AtlasSpriteDirect sprite = new AtlasSpriteDirect(name);
        try {
            // Copied almost directly from TextureMap.
            PngSizeInfo pngsizeinfo = PngSizeInfo.func_188532_a(manager.func_110536_a(location));
            try (IResource iresource = manager.func_110536_a(location)) {
                boolean flag = iresource.func_110526_a("animation") != null;
                sprite.func_188538_a(pngsizeinfo, flag);
                sprite.func_188539_a(iresource, Minecraft.func_71410_x().field_71474_y.field_151442_I + 1);
                return sprite;
            }
        } catch (IOException io) {
            if (careIfMissing) {
                // Do the same as forge - track the missing texture for later rather than printing out the error.
                FMLClientHandler.instance().trackMissingTexture(location);
            }
            return null;
        }
    }

    @Override
    public boolean hasCustomLoader(IResourceManager manager, ResourceLocation location) {
        return true;
    }

    @Override
    public void func_147963_d(int level) {
        if (current != null) {
            current.func_147963_d(level);
        }
    }

    // Overrides

    @Override
    public int func_110970_k() {
        if (current == null) {
            return 0;
        }
        return current.func_110970_k();
    }

    @Override
    public void func_94217_a(TextureAtlasSprite from) {
        super.func_94217_a(from);
        if (current == null) {
            current = from;
        } else {
            current.func_94217_a(from);
        }
    }

    @Override
    public int[][] func_147965_a(int index) {
        if (current == null) {
            return new int[1][1];
        }
        return current.func_147965_a(index);
    }

    @Override
    public void func_110968_a(List<int[][]> newFramesTextureData) {
        // NO-OP
    }
}
