/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.transport.recipe;

import net.minecraft.inventory.InventoryCrafting;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.IRecipe;
import net.minecraft.util.NonNullList;
import net.minecraft.world.World;

import net.minecraftforge.common.ForgeHooks;

import buildcraft.lib.misc.StackUtil;
import buildcraft.lib.recipe.ChangingItemStack;
import buildcraft.lib.recipe.IRecipeViewable;

import buildcraft.transport.BCTransportItems;
import buildcraft.transport.item.ItemPluggableFacade;
import buildcraft.transport.plug.FacadeBlockStateInfo;
import buildcraft.transport.plug.FacadeInstance;
import buildcraft.transport.plug.FacadeStateManager;

public enum FacadeSwapRecipe implements IRecipe, IRecipeViewable.IViewableGrid {
    INSTANCE;

    private static final int TIME_GAP = 500;

    private static final ChangingItemStack[] INPUTS = { null };
    private static ChangingItemStack OUTPUTS;

    public static void genRecipes() {
        NonNullList<ItemStack> list1 = NonNullList.func_191196_a();
        NonNullList<ItemStack> list2 = NonNullList.func_191196_a();
        for (FacadeBlockStateInfo info : FacadeStateManager.validFacadeStates.values()) {
            if (info.isVisible) {
                ItemStack stack = createFacade(info, false);
                ItemStack stackHollow = createFacade(info, true);
                list1.add(stack);
                list1.add(stackHollow);
                list2.add(stackHollow);
                list2.add(stack);
            }
        }
        INPUTS[0] = new ChangingItemStack(list1);
        INPUTS[0].setTimeGap(TIME_GAP);

        OUTPUTS = new ChangingItemStack(list2);
        OUTPUTS.setTimeGap(TIME_GAP);
    }

    @Override
    public boolean func_77569_a(InventoryCrafting inv, World world) {
        return !func_77572_b(inv).func_190926_b();
    }

    @Override
    public ItemStack func_77572_b(InventoryCrafting inv) {
        ItemStack stackIn = StackUtil.EMPTY;
        for (int s = 0; s < inv.func_70302_i_(); s++) {
            ItemStack stack = inv.func_70301_a(s);
            if (!stack.func_190926_b()) {
                if (stackIn.func_190926_b()) {
                    stackIn = stack;
                } else {
                    return StackUtil.EMPTY;
                }
            }
        }
        if (stackIn.func_77973_b() != BCTransportItems.plugFacade) {
            return StackUtil.EMPTY;
        }
        FacadeInstance states = ItemPluggableFacade.getStates(stackIn);
        states = states.withSwappedIsHollow();
        return BCTransportItems.plugFacade.createItemStack(states);
    }

    @Override
    public int func_77570_a() {
        return 1;
    }

    @Override
    public ItemStack func_77571_b() {
        return StackUtil.EMPTY;
    }

    @Override
    public NonNullList<ItemStack> func_179532_b(InventoryCrafting inv) {
        return ForgeHooks.defaultRecipeGetRemainingItems(inv);
    }

    @Override
    public ChangingItemStack[] getRecipeInputs() {
        if (INPUTS[0] == null) {
            genRecipes();
        }
        return INPUTS;
    }

    @Override
    public ChangingItemStack getRecipeOutputs() {
        if (OUTPUTS == null) {
            genRecipes();
        }
        return OUTPUTS;
    }

    private static ItemStack createFacade(FacadeBlockStateInfo info, boolean isHollow) {
        FacadeInstance state = FacadeInstance.createSingle(info, isHollow);
        return BCTransportItems.plugFacade.createItemStack(state);
    }

    @Override
    public int getRecipeWidth() {
        return 1;
    }

    @Override
    public int getRecipeHeight() {
        return 1;
    }
}
